/*
 * skin effect samples programs
 * 2016-09 Kouichi Hirabayashi
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <complex.h>
#include "Complex.h"

#define I0(x)	In(0, x)
#define I1(x)	In(1, x)
#define J0(x)	Jn(0, x)
#define J1(x)	Jn(1, x)
#define K0(x)	Kn(0, x)
#define K1(x)	Kn(1, x)

int solid_cd();
int solid();
int tube();
int coax_outer();
int Zt();
int bimetal();

int main(argc, argv) char **argv;
{
// Please rmove the comment of the required program.
//  solid_cd();		// current distribution of isolated solid conndouctor
  solid();		// skineffect of isolated solid conductor
//  tube();		// skin effect of isolated tube
//  coax_outer();	// skini effect of outer connductor of the coaxial cable 
//  bimetal();		// skineffect of bimetallic condouctor
//  Zt();		// transfer impedance of outer connductor of the coaxial cable
  exit(0);
}

// current distribution of solid conductor

int solid_cd()
{
  double a, f, u, g, r, PI;
  COMPLEX k, z;

  PI = atan2(0.0, -1.0);
  a = 5e-4;		// radius of solid copper conductor
  u = 4e-7 * PI;	// permeability
  g = 5.80e7;		// conductivity
  f = 1e6;		// frequency
  for (r = 0.0; r <= a * 1.001; r += a * 0.01) {
	k = csqrt(I * 2.0 * PI * f * u * g);
	z = I0(r * k) / I0(a * k);
	printf("%g %g %g %g\n", r, cabs(z), creal(z), cimag(z));
  }
  return 0;
}

// skin effect of solid conductor

int solid()
{
  double a, f, u, g, k, rdc, x, PI;
  COMPLEX A, z;

  PI = atan2(0.0, -1.0);
  a = 5e-4;		// radius of solid copper conductor
  u = 4e-7 * PI;	// permeability
  g = 5.80e7;		// conductivity

  rdc = 1.0 / (g * PI * a * a);		// DC registance
  fprintf(stderr, "# Rdc(%g)\n", rdc);
  for (f = 1e2; f <= 1e9; f *= 1.01) {	// frequency
	k = sqrt(2.0 * PI * f * u * g);
	A = csqrt(I * a * k);
	x = cabs(A);
	if (x < 1e-6)
		z = (COMPLEX)1.0;
	else if (1e3 < x)
		z = A * 0.5;
	else
		z = I0(A) * (A / I1(A)) * 0.5;	// Zac/Rdc
	printf("%g %g %g\n", f, creal(z), cimag(z));
  }
  return 0;
}


// skin effect of isolated tubular conductor (inner conductor of coaxial cable)

int tube()
{
  double a, b, f, u, g, k, rdc, x, PI;
  COMPLEX A, B, SQRTi, N, D, z;

  PI = atan2(0.0, -1.0);
  a = 3e-3;
  b = 2.6e-3;
  u = 4e-7 * PI;
  g = 5.80e7;
  rdc = 1.0 / (g * PI * (a * a - b * b));
  fprintf(stderr, "# Rdc(%g)\n", rdc);
  SQRTi = csqrt(I);
  for (f = 1e2; f <= 1e9; f *= 1.01) {
	k = sqrt(2.0 * PI * f * u * g);
	A = SQRTi * a * k;
	B = SQRTi * b * k;
	x = a * k;
	if (x == 0.0)
		z = (COMPLEX)1.0;
	else if (9e2 < x) {
		k = 1.0 - (b / a) * (b / a);
		z = A * 0.5 * k;
	}
	else {
		N = I0(A) + I1(B) * K0(A) / K1(B);
		D = I1(A) - I1(B) * K1(A) / K1(B);
		z = (N / D) * A * (1 - b / a * b / a) * 0.5;
	}
	printf("%g %g %g\n", f, creal(z), cimag(z));
  }
  return 0;
}


// impedance of outer conductor of coaxial cable

int coax_outer()
{
  double a, b, u, g, f, x, rdc, PI;
  COMPLEX A, B, z;

  PI = atan2(0.0, -1.0);
  a = 1.85e-3;
  b = 1.5e-3;
  u = 4e-7 * PI;
  g = 5.80e7;
  rdc = 1.0 / (g * PI * (a * a - b * b));
  // impedance = R + j*w*L (outer conductor)
  for (f = 1e1; f <= 1e9; f *= 1.01) {
	A = a * csqrt(I * 2.0 * PI * f * u * g);
	B = b * csqrt(I * 2.0 * PI * f * u * g);
	x = cabs(A);
	if (x == 0.0)
		z = (COMPLEX)1.0;
	else if (1e3 < x)
		z = B * ((a / b) * (a / b) - 1.0) * 0.5;
	else {
		z = (I0(B) * K1(A) + I1(A) * K0(B))
			/ (I1(A) * K1(B) - I1(B) * K1(A))
			* B * ((a / b) * (a / b) - 1.0) * 0.5;
	}
	printf("%g %g %g\n", f, creal(z), cimag(z));
  }
  return 0;
}

// transfer impedanve = V / I (coaxial cable outer conductor)

int Zt()
{
  double a, b, u, g, f, x, rdc, PI;
  COMPLEX A, B, z, z1, z2;

  a = 1.85e-3;
  b = 1.5e-3;
  PI = atan2(0.0, -1.0);
  u = 4e-7 * PI;
  g = 5.80e7;
  rdc = 1.0 / (g * PI * (a * a - b * b));
  for (f = 1e2; f <= 1.0e9; f *= 1.01) {
	A = a * csqrt(I * 2.0 * PI * f * u * g);
	B = b * csqrt(I * 2.0 * PI * f * u * g);
	x = cabs(B);
	if (5e2< x)
		z1 = 0.0;
	else {
		// I/(i0/g)
		z1 = (I1(A) * K1(B) - I1(B) * K1(A))
			/ (I0(B) * K1(A) + I1(A) * K0(B))
			/ B * 2.0 * PI * b * b;
		// i1/i0
		z2 = (I0(A) * K1(A) + I1(A) * K0(A))
			/ (I0(B) * K1(A) + I1(A) * K0(B));
		// Zt
		z1 = z2 / z1 / g;
	}
	printf("%g %g\n", f, cabs(z1));
//	printf("%g %g %g\n", f, cabs(z1), carg(z1));
//	printf("%g %g %g %g\n", f, cabs(z1), creal(z1), cimag(z1));
  }
  return 0;
}

// bimetallic conductor

int bimetal()
{
  double a, b, f, u1, g1, u2, g2, k, rdc, x, PI;
  COMPLEX A, B, C, N, D, z;

	
  a = 0.1303e-2;
  b = 0.1034e-2;
  PI = atan2(0.0, -1.0);
  g2 = 1.0 / 27.91e-8;
  g1 = 1.0 / 1.830e-8;
  u2 = 78.8 * 4e-7 * PI;
  u1 = 4e-7 * PI;
  rdc = 1.0 / (g1 * PI * (a - b) * (a - b) + g2 * PI * b * b);
  for (f = 1; f <= 1e9; f *= 1.01) {
	B = csqrt(I * 2.0 * PI * f * u1 * g1);
	A = a * B;
	B *= b;
	C = b * csqrt(I * 2.0 * PI * f * u2 * g2);
	x = a * sqrt(2.0 * PI * f * u1 * g1);
	if (x == 0.0)
		z = (COMPLEX)1.0;
	else if (3e2 < x)
		z = (1.0 - (b / a) * (b / a) * (1.0 - g2 / g1)) * A * 0.5;
	else {
		k = sqrt((u1 / u2) * (g2 / g1));
		N = I0(A) + I1(B) * K0(A) / K1(B)
			+ (I0(A) * K0(B) / K1(B) - I0(B) * K0(A) / K1(B))
				* I1(C) / I0(C) * k;
		D = I1(A) - I1(B) * K1(A) / K1(B)
			+ (I1(A) * K0(B) / K1(B) + I0(B) * K1(A) / K1(B))
				* I1(C) / I0(C) * k;
		z = (1.0 - (b / a) * (b / a) * (1.0 - g2 / g1))
			* N / D * A * 0.5;
	}
	printf("%g %g %g\n", f, creal(z), cimag(z));
  }
  return 0;
}
